#include "stdafx.h"
#include "VendorCmd.h"

CVendorCmd::CVendorCmd(void)
{
	m_bIsLoaded			= FALSE;
	m_hMXDevInst	= NULL;
	m_pfnI2CBusRead		= NULL;
	m_pfnI2CBusWrite	= NULL;
}

CVendorCmd::~CVendorCmd(void)
{
}


BOOL CVendorCmd::Load()
{
	BOOL bSuccess = FALSE;

	do
	{
		if(!m_hMXDevInst)
			m_hMXDevInst = ::LoadLibrary(I2C_INF_DLL_FILE);

		if (!m_hMXDevInst)
			break;
		
		if(m_hMXDevInst)
		{
			m_pfnI2CBusRead		= (PCII2CBUSREAD)GetProcAddress(m_hMXDevInst, "I2CBusRead");
			m_pfnI2CBusWrite	= (PCII2CBUSWRITE)GetProcAddress(m_hMXDevInst, "I2CBusWrite");
		}
		if (!m_pfnI2CBusRead || !m_pfnI2CBusWrite)
			break;
		bSuccess = TRUE;
	} while (FALSE);

	m_bIsLoaded			= TRUE;

	return bSuccess;
}

void CVendorCmd::Unload()
{
	::FreeLibrary(m_hMXDevInst);
	m_hMXDevInst	= NULL;
	m_pfnI2CBusRead		= NULL;
	m_pfnI2CBusWrite	= NULL;
	m_bIsLoaded			= FALSE;
}

BOOL CVendorCmd::MX_ReadI2C(DWORD dwAddr, DWORD dwSubAddr, BYTE* pData, ULONG ulReadLen)
{
	if (!pData || !m_pfnI2CBusRead)
		return FALSE;

	//slave 	addr of the device
	DWORD dwSlaveAddr = dwAddr;

	//length of the sub address
	ULONG nSizeOfSubAddr = 1;

	//if the addr is 10-bit width
	//normally it is FALSE
	BOOL bIs10bitAddr = FALSE;

	//interface id code
	DWORD dwI2CInfID = I2C_INF_ID;

	m_pfnI2CBusRead(pData, dwSlaveAddr, dwSubAddr, nSizeOfSubAddr, 
					ulReadLen, bIs10bitAddr, dwI2CInfID);

	return TRUE;
}

BOOL CVendorCmd::MX_WriteI2C(DWORD dwAddr, DWORD dwSubAddr, BYTE* pData, ULONG ulWriteLen)
{
	if (!pData || !m_pfnI2CBusWrite)
		return FALSE;

	//slave 	addr of the device
	DWORD dwSlaveAddr = dwAddr;

	//length of the sub address
	ULONG nSizeOfSubAddr = 1;

	//if the addr is 10-bit width
	//normally it is FALSE
	BOOL bIs10bitAddr = FALSE;

	//interface id code
	DWORD dwI2CInfID = I2C_INF_ID;

	m_pfnI2CBusWrite(pData, dwSlaveAddr, dwSubAddr, nSizeOfSubAddr, 
					ulWriteLen, bIs10bitAddr, dwI2CInfID);

	return TRUE;
}

BYTE CVendorCmd::ConvertHexByteStrToHex(WCHAR* wszVal)
{
	int iStrLen = wcslen(wszVal);
	int j = 0;
	BYTE iVal = 0;

	for (int i = iStrLen - 1; i >= 0; i --)
	{
		iVal += ConvertHexCharToHex(wszVal[i]) << (j * 4);
		j ++;
	}

	return iVal;
}

BYTE CVendorCmd::ConvertHexCharToHex(WCHAR chVal)
{
	// Valid return value [0, 15].
	// 255 = Invalid.

	BYTE byVal;

	if ((chVal >= L'0') && (chVal <= L'9'))
	{
		byVal = chVal - '0';
	}
	else if ((chVal >= L'a') && (chVal <= L'f'))
	{
		byVal = chVal - L'a' + 10;
	}
	else if ((chVal >= L'A') && (chVal <= L'F'))
	{
		byVal = chVal - L'A' + 10;
	}
	else
	{
		byVal = 255;
	}

	return byVal;
}
